@@ -19,12 +19,13 @@
 #include <mm/core_mmu.h>
 #include <mm/fobj.h>
 #include <mm/mobj.h>
+#include <mm/phys_mem.h>
 #include <mm/vm.h>
 #include <optee_ffa.h>
 #include <stdio.h>
 #include <string.h>
-#include <tee_api_types.h>
 #include <tee/uuid.h>
+#include <tee_api_types.h>
 #include <trace.h>
 #include <types_ext.h>
 #include <utee_defines.h>
@@ -37,6 +38,7 @@
 #define SP_MANIFEST_ATTR_WRITE		BIT(1)
 #define SP_MANIFEST_ATTR_EXEC		BIT(2)
 #define SP_MANIFEST_ATTR_NSEC		BIT(3)
+#define SP_MANIFEST_ATTR_GP		BIT(4)
 
 #define SP_MANIFEST_ATTR_RO		(SP_MANIFEST_ATTR_READ)
 #define SP_MANIFEST_ATTR_RW		(SP_MANIFEST_ATTR_READ | \
@@ -53,6 +55,16 @@
 #define SP_MANIFEST_NS_INT_MANAGED_EXIT	(0x1)
 #define SP_MANIFEST_NS_INT_SIGNALED	(0x2)
 
+#define SP_MANIFEST_EXEC_STATE_AARCH64	(0x0)
+#define SP_MANIFEST_EXEC_STATE_AARCH32	(0x1)
+
+#define SP_MANIFEST_DIRECT_REQ_RECEIVE	BIT(0)
+#define SP_MANIFEST_DIRECT_REQ_SEND	BIT(1)
+#define SP_MANIFEST_INDIRECT_REQ	BIT(2)
+
+#define SP_MANIFEST_VM_CREATED_MSG	BIT(0)
+#define SP_MANIFEST_VM_DESTROYED_MSG	BIT(1)
+
 #define SP_PKG_HEADER_MAGIC (0x474b5053)
 #define SP_PKG_HEADER_VERSION_V1 (0x1)
 #define SP_PKG_HEADER_VERSION_V2 (0x2)
@@ -119,8 +131,6 @@ TEE_Result sp_partition_info_get(uint32_t ffa_vers, void *buf, size_t buf_size,
 				 bool count_only)
 {
 	TEE_Result res = TEE_SUCCESS;
-	uint32_t part_props = FFA_PART_PROP_DIRECT_REQ_RECV |
-			      FFA_PART_PROP_DIRECT_REQ_SEND;
 	struct sp_session *s = NULL;
 
 	TAILQ_FOREACH(s, &open_sp_sessions, link) {
@@ -137,7 +147,7 @@ TEE_Result sp_partition_info_get(uint32_t ffa_vers, void *buf, size_t buf_size,
 			res = spmc_fill_partition_entry(ffa_vers, buf, buf_size,
 							*elem_count,
 							s->endpoint_id, 1,
-							part_props, uuid_words);
+							s->props, uuid_words);
 		}
 		*elem_count += 1;
 	}
@@ -214,6 +224,10 @@ static TEE_Result sp_create_ctx(const TEE_UUID *bin_uuid, struct sp_session *s)
 
 	set_sp_ctx_ops(&spc->ts_ctx);
 
+#ifdef CFG_TA_PAUTH
+	crypto_rng_read(&spc->uctx.keys, sizeof(spc->uctx.keys));
+#endif
+
 	return TEE_SUCCESS;
 
 err:
@@ -257,6 +271,9 @@ static TEE_Result sp_create_session(struct sp_sessions_head *open_sessions,
 
 	s->boot_order = boot_order;
 
+	/* Other properties are filled later, based on the SP's manifest */
+	s->props = FFA_PART_PROP_IS_PE_ID;
+
 	res = new_session_id(&s->endpoint_id);
 	if (res)
 		goto err;
@@ -518,7 +535,7 @@ static TEE_Result load_binary_sp(struct ts_session *s,
 	bin_page_count = bin_size_rounded / SMALL_PAGE_SIZE;
 
 	/* Allocate memory */
-	mm = tee_mm_alloc(&tee_mm_sec_ddr, bin_size_rounded);
+	mm = phys_mem_ta_alloc(bin_size_rounded);
 	if (!mm) {
 		res = TEE_ERROR_OUT_OF_MEMORY;
 		goto err;
@@ -880,6 +897,15 @@ static TEE_Result handle_fdt_load_relative_mem_regions(struct sp_ctx *ctx,
 			return TEE_ERROR_BAD_FORMAT;
 		}
 
+		if (IS_ENABLED(CFG_TA_BTI) &&
+		    attributes & SP_MANIFEST_ATTR_GP) {
+			if (!(attributes & SP_MANIFEST_ATTR_RX)) {
+				EMSG("Guard only executable region");
+				return TEE_ERROR_BAD_FORMAT;
+			}
+			perm |= TEE_MATTR_GUARDED;
+		}
+
 		res = sp_dt_get_u32(fdt, subnode, "load-flags", &flags);
 		if (res != TEE_SUCCESS && res != TEE_ERROR_ITEM_NOT_FOUND) {
 			EMSG("Optional field with invalid value: flags");
@@ -901,7 +927,7 @@ static TEE_Result handle_fdt_load_relative_mem_regions(struct sp_ctx *ctx,
 			struct mobj *m = NULL;
 			unsigned int idx = 0;
 
-			mm = tee_mm_alloc(&tee_mm_sec_ddr, size);
+			mm = phys_mem_ta_alloc(size);
 			if (!mm)
 				return TEE_ERROR_OUT_OF_MEMORY;
 
@@ -1209,6 +1235,15 @@ static TEE_Result handle_fdt_mem_regions(struct sp_ctx *ctx, void *fdt)
 			return TEE_ERROR_BAD_FORMAT;
 		}
 
+		if (IS_ENABLED(CFG_TA_BTI) &&
+		    attributes & SP_MANIFEST_ATTR_GP) {
+			if (!(attributes & SP_MANIFEST_ATTR_RX)) {
+				EMSG("Guard only executable region");
+				return TEE_ERROR_BAD_FORMAT;
+			}
+			perm |= TEE_MATTR_GUARDED;
+		}
+
 		/*
 		 * The SP is a secure endpoint, security attribute can be
 		 * secure or non-secure.
@@ -1225,7 +1260,7 @@ static TEE_Result handle_fdt_mem_regions(struct sp_ctx *ctx, void *fdt)
 
 		if (alloc_needed) {
 			/* Base address is missing, we have to allocate */
-			mm = tee_mm_alloc(&tee_mm_sec_ddr, size);
+			mm = phys_mem_ta_alloc(size);
 			if (!mm)
 				return TEE_ERROR_OUT_OF_MEMORY;
 
@@ -1377,6 +1412,22 @@ static TEE_Result handle_hw_features(void *fdt)
 			return res;
 	}
 
+	/* Modify the property only if it's already present */
+	if (!sp_dt_get_u32(fdt, node, "bti", &val)) {
+		res = fdt_setprop_u32(fdt, node, "bti",
+				      feat_bti_is_implemented());
+		if (res)
+			return res;
+	}
+
+	/* Modify the property only if it's already present */
+	if (!sp_dt_get_u32(fdt, node, "pauth", &val)) {
+		res = fdt_setprop_u32(fdt, node, "pauth",
+				      feat_pauth_is_implemented());
+		if (res)
+			return res;
+	}
+
 	return TEE_SUCCESS;
 }
 
@@ -1432,6 +1483,80 @@ static TEE_Result read_ffa_version(const void *fdt, struct sp_session *s)
 	return TEE_SUCCESS;
 }
 
+static TEE_Result read_sp_exec_state(const void *fdt, struct sp_session *s)
+{
+	TEE_Result res = TEE_ERROR_BAD_PARAMETERS;
+	uint32_t exec_state = 0;
+
+	res = sp_dt_get_u32(fdt, 0, "execution-state", &exec_state);
+	if (res) {
+		EMSG("Mandatory property is missing: execution-state");
+		return res;
+	}
+
+	/* Currently only AArch64 SPs are supported */
+	if (exec_state == SP_MANIFEST_EXEC_STATE_AARCH64) {
+		s->props |= FFA_PART_PROP_AARCH64_STATE;
+	} else {
+		EMSG("Invalid execution-state value: %"PRIu32, exec_state);
+		return TEE_ERROR_BAD_PARAMETERS;
+	}
+
+	return TEE_SUCCESS;
+}
+
+static TEE_Result read_sp_msg_types(const void *fdt, struct sp_session *s)
+{
+	TEE_Result res = TEE_ERROR_BAD_PARAMETERS;
+	uint32_t msg_method = 0;
+
+	res = sp_dt_get_u32(fdt, 0, "messaging-method", &msg_method);
+	if (res) {
+		EMSG("Mandatory property is missing: messaging-method");
+		return res;
+	}
+
+	if (msg_method & SP_MANIFEST_DIRECT_REQ_RECEIVE)
+		s->props |= FFA_PART_PROP_DIRECT_REQ_RECV;
+
+	if (msg_method & SP_MANIFEST_DIRECT_REQ_SEND)
+		s->props |= FFA_PART_PROP_DIRECT_REQ_SEND;
+
+	if (msg_method & SP_MANIFEST_INDIRECT_REQ)
+		IMSG("Indirect messaging is not supported");
+
+	return TEE_SUCCESS;
+}
+
+static TEE_Result read_vm_availability_msg(const void *fdt,
+					   struct sp_session *s)
+{
+	TEE_Result res = TEE_ERROR_BAD_PARAMETERS;
+	uint32_t v = 0;
+
+	res = sp_dt_get_u32(fdt, 0, "vm-availability-messages", &v);
+
+	/* This field in the manifest is optional */
+	if (res == TEE_ERROR_ITEM_NOT_FOUND)
+		return TEE_SUCCESS;
+
+	if (res)
+		return res;
+
+	if (v & ~(SP_MANIFEST_VM_CREATED_MSG | SP_MANIFEST_VM_DESTROYED_MSG)) {
+		EMSG("Invalid vm-availability-messages value: %"PRIu32, v);
+		return TEE_ERROR_BAD_PARAMETERS;
+	}
+
+	if (v & SP_MANIFEST_VM_CREATED_MSG)
+		s->props |= FFA_PART_PROP_NOTIF_CREATED;
+
+	if (v & SP_MANIFEST_VM_DESTROYED_MSG)
+		s->props |= FFA_PART_PROP_NOTIF_DESTROYED;
+
+	return TEE_SUCCESS;
+}
+
 static TEE_Result sp_init_uuid(const TEE_UUID *bin_uuid, const void * const fdt)
 {
 	TEE_Result res = TEE_SUCCESS;
@@ -1475,6 +1600,18 @@ static TEE_Result sp_init_uuid(const TEE_UUID *bin_uuid, const void * const fdt)
 	if (res)
 		return res;
 
+	res = read_sp_exec_state(fdt, sess);
+	if (res)
+		return res;
+
+	res = read_sp_msg_types(fdt, sess);
+	if (res)
+		return res;
+
+	res = read_vm_availability_msg(fdt, sess);
+	if (res)
+		return res;
+
 	return TEE_SUCCESS;
 }
 
@@ -1564,6 +1701,10 @@ TEE_Result sp_enter(struct thread_smc_args *args, struct sp_session *sp)
 	ctx->sp_regs.x[5] = args->a5;
 	ctx->sp_regs.x[6] = args->a6;
 	ctx->sp_regs.x[7] = args->a7;
+#ifdef CFG_TA_PAUTH
+	ctx->sp_regs.apiakey_hi = ctx->uctx.keys.apia_hi;
+	ctx->sp_regs.apiakey_lo = ctx->uctx.keys.apia_lo;
+#endif
 
 	res = sp->ts_sess.ctx->ops->enter_invoke_cmd(&sp->ts_sess, 0);
 
@@ -1730,7 +1871,7 @@ static const struct ts_ops sp_ops = {
 
 static TEE_Result process_sp_pkg(uint64_t sp_pkg_pa, TEE_UUID *sp_uuid)
 {
-	enum teecore_memtypes mtype = MEM_AREA_TA_RAM;
+	enum teecore_memtypes mtype = MEM_AREA_SEC_RAM_OVERALL;
 	struct sp_pkg_header *sp_pkg_hdr = NULL;
 	struct fip_sp *sp = NULL;
 	uint64_t sp_fdt_end = 0;
