@@ -523,6 +523,12 @@ TEE_Result spmc_fill_partition_entry(uint32_t ffa_vers, void *buf, size_t blen,
 
 	fpi->partition_properties = part_props;
 
+	/* In FF-A 1.0 only bits [2:0] are defined, let's mask others */
+	if (ffa_vers < FFA_VERSION_1_1)
+		fpi->partition_properties &= FFA_PART_PROP_DIRECT_REQ_RECV |
+					     FFA_PART_PROP_DIRECT_REQ_SEND |
+					     FFA_PART_PROP_INDIRECT_MSGS;
+
 	if (ffa_vers >= FFA_VERSION_1_1) {
 		if (uuid_words)
 			memcpy(fpi->uuid, uuid_words, FFA_UUID_SIZE);
@@ -792,6 +798,8 @@ static void handle_blocking_call(struct thread_smc_args *args,
 		sec_caps = OPTEE_FFA_SEC_CAP_ARG_OFFSET;
 		if (spmc_notif_is_ready)
 			sec_caps |= OPTEE_FFA_SEC_CAP_ASYNC_NOTIF;
+		if (IS_ENABLED(CFG_RPMB_ANNOUNCE_PROBE_CAP))
+			sec_caps |= OPTEE_FFA_SEC_CAP_RPMB_PROBE;
 		spmc_set_args(args, direct_resp_fid,
 			      swap_src_dst(args->a1), 0, 0,
 			      THREAD_RPC_MAX_NUM_PARAMS, sec_caps);
