@@ -288,40 +288,6 @@ const char *stm32mp_pmic_get_cpu_supply_name(void)
 	return cpu_supply_name;
 }
 
-/* Preallocate not that much regu references */
-static char *nsec_access_regu_name[PMIC_REGU_COUNT];
-
-bool stm32mp_nsec_can_access_pmic_regu(const char *name)
-{
-	size_t n = 0;
-
-	for (n = 0; n < ARRAY_SIZE(nsec_access_regu_name); n++)
-		if (nsec_access_regu_name[n] &&
-		    !strcmp(nsec_access_regu_name[n], name))
-			return true;
-
-	return false;
-}
-
-static void register_nsec_regu(const char *name_ref)
-{
-	size_t n = 0;
-
-	assert(!stm32mp_nsec_can_access_pmic_regu(name_ref));
-
-	for (n = 0; n < ARRAY_SIZE(nsec_access_regu_name); n++) {
-		if (!nsec_access_regu_name[n]) {
-			nsec_access_regu_name[n] = strdup(name_ref);
-
-			if (!nsec_access_regu_name[n])
-				panic();
-			break;
-		}
-	}
-
-	assert(stm32mp_nsec_can_access_pmic_regu(name_ref));
-}
-
 static TEE_Result pmic_set_state(struct regulator *regulator, bool enable)
 {
 	struct pmic_regulator_data *priv = regulator->priv;
@@ -389,14 +355,6 @@ static TEE_Result pmic_set_voltage(struct regulator *regulator, int level_uv)
 	return TEE_SUCCESS;
 }
 
-static int cmp_int_value(const void *a, const void *b)
-{
-	const int *ia = a;
-	const int *ib = b;
-
-	return CMP_TRILEAN(*ia, *ib);
-}
-
 static size_t refine_levels_array(size_t count, int *levels_uv,
 				  int min_uv, int max_uv)
 {
@@ -404,7 +362,7 @@ static size_t refine_levels_array(size_t count, int *levels_uv,
 	size_t m = 0;
 
 	/* We need to sort the array has STPMIC1 driver does not */
-	qsort(levels_uv, count, sizeof(*levels_uv), cmp_int_value);
+	qsort_int(levels_uv, count);
 
 	/* Remove duplicates and return optimized count */
 	for (n = 1; n < count; n++) {
@@ -550,8 +508,10 @@ static TEE_Result release_voltage_lists(void)
 	for (n = 0; n < ARRAY_SIZE(pmic_regulators); n++) {
 		struct pmic_regulator_data *priv = pmic_regulators[n].priv;
 
-		if (priv && priv->voltages_level)
+		if (priv && priv->voltages_level) {
 			free(priv->voltages_level);
+			priv->voltages_level = NULL;
+		}
 	}
 
 	return TEE_SUCCESS;
@@ -637,9 +597,6 @@ static void parse_regulator_fdt_nodes(const void *fdt, int pmic_node)
 
 		assert(stpmic1_regulator_is_valid(regu_name));
 
-		if (status & DT_STATUS_OK_NSEC)
-			register_nsec_regu(regu_name);
-
 		for (n = 0; n < ARRAY_SIZE(regu_lp_state); n++)
 			dt_get_regu_low_power_config(fdt, regu_name, regu_node,
 						     regu_lp_state[n].name);
@@ -713,29 +670,6 @@ void stm32mp_put_pmic(void)
 	stm32_i2c_suspend(i2c_handle);
 }
 
-static void register_non_secure_pmic(void)
-{
-	/* Allow this function to be called when STPMIC1 not used */
-	if (!i2c_handle->base.pa)
-		return;
-
-	stm32mp_register_non_secure_pinctrl(i2c_handle->pinctrl);
-	if (i2c_handle->pinctrl_sleep)
-		stm32mp_register_non_secure_pinctrl(i2c_handle->pinctrl_sleep);
-
-	stm32mp_register_non_secure_periph_iomem(i2c_handle->base.pa);
-}
-
-static void register_secure_pmic(void)
-{
-	stm32mp_register_secure_pinctrl(i2c_handle->pinctrl);
-	if (i2c_handle->pinctrl_sleep)
-		stm32mp_register_secure_pinctrl(i2c_handle->pinctrl_sleep);
-
-	stm32mp_register_secure_periph_iomem(i2c_handle->base.pa);
-	register_pm_driver_cb(pmic_pm, NULL, "stm32mp1-pmic");
-}
-
 static TEE_Result initialize_pmic(const void *fdt, int pmic_node)
 {
 	unsigned long pmic_version = 0;
@@ -753,9 +687,7 @@ static TEE_Result initialize_pmic(const void *fdt, int pmic_node)
 	stm32mp_put_pmic();
 
 	if (dt_pmic_is_secure())
-		register_secure_pmic();
-	else
-		register_non_secure_pmic();
+		register_pm_driver_cb(pmic_pm, NULL, "stm32mp1-pmic");
 
 	parse_regulator_fdt_nodes(fdt, pmic_node);
 
