@@ -6,6 +6,7 @@
 #include <config.h>
 #include <console.h>
 #include <drivers/gic.h>
+#include <drivers/stm32_rif.h>
 #include <drivers/stm32_uart.h>
 #include <initcall.h>
 #include <kernel/boot.h>
@@ -64,18 +65,17 @@ void plat_console_init(void)
 	/* Early console initialization before MMU setup */
 	struct uart {
 		paddr_t pa;
-		bool secure;
 	} uarts[] = {
 		[0] = { .pa = 0 },
-		[1] = { .pa = USART1_BASE, .secure = true, },
-		[2] = { .pa = USART2_BASE, .secure = false, },
-		[3] = { .pa = USART3_BASE, .secure = false, },
-		[4] = { .pa = UART4_BASE, .secure = false, },
-		[5] = { .pa = UART5_BASE, .secure = false, },
-		[6] = { .pa = USART6_BASE, .secure = false, },
-		[7] = { .pa = UART7_BASE, .secure = false, },
-		[8] = { .pa = UART8_BASE, .secure = false, },
-		[9] = { .pa = UART9_BASE, .secure = false, },
+		[1] = { .pa = USART1_BASE },
+		[2] = { .pa = USART2_BASE },
+		[3] = { .pa = USART3_BASE },
+		[4] = { .pa = UART4_BASE },
+		[5] = { .pa = UART5_BASE },
+		[6] = { .pa = USART6_BASE },
+		[7] = { .pa = UART7_BASE },
+		[8] = { .pa = UART8_BASE },
+		[9] = { .pa = UART9_BASE },
 	};
 
 	static_assert(ARRAY_SIZE(uarts) > CFG_STM32_EARLY_CONSOLE_UART);
@@ -85,7 +85,6 @@ void plat_console_init(void)
 
 	/* No clock yet bound to the UART console */
 	console_data.clock = NULL;
-	console_data.secure = uarts[CFG_STM32_EARLY_CONSOLE_UART].secure;
 	stm32_uart_init(&console_data, uarts[CFG_STM32_EARLY_CONSOLE_UART].pa);
 	register_serial_console(&console_data.chip);
 
@@ -123,7 +122,7 @@ static TEE_Result init_console_from_dt(void)
 	console_flush();
 	console_data = *pd;
 	register_serial_console(&console_data.chip);
-	IMSG("DTB enables console (%ssecure)", pd->secure ? "" : "non-");
+	IMSG("DTB enables console");
 	free(pd);
 
 	return TEE_SUCCESS;
@@ -149,3 +148,27 @@ void boot_secondary_init_intc(void)
 {
 	gic_init_per_cpu();
 }
+
+#ifdef CFG_STM32_RIF
+void stm32_rif_access_violation_action(void)
+{
+}
+#endif /* CFG_STM32_RIF */
+
+bool stm32mp_allow_probe_shared_device(const void *fdt, int node)
+{
+	static int uart_console_node = -1;
+	static bool once;
+
+	if (!once) {
+		get_console_node_from_dt((void *)fdt, &uart_console_node,
+					 NULL, NULL);
+		once = true;
+	}
+
+	/* Allow OP-TEE console to be shared with non-secure world */
+	if (node == uart_console_node)
+		return true;
+
+	return false;
+}
