@@ -135,7 +135,7 @@ static unsigned long core_mmu_ptp_create(unsigned long ppn)
 
 static unsigned long core_mmu_pte_ppn(struct mmu_pte *pte)
 {
-	return pte->entry >> PTE_PPN_SHIFT;
+	return (pte->entry & PTE_PPN) >> PTE_PPN_SHIFT;
 }
 
 static unsigned long pa_to_ppn(paddr_t pa)
@@ -152,9 +152,10 @@ static unsigned long core_mmu_pgt_to_satp(unsigned long asid,
 					  struct mmu_pgt *pgt)
 {
 	unsigned long satp = 0;
-	unsigned long pgt_ppn = virt_to_phys(pgt) >> RISCV_PGSHIFT;
+	unsigned long pgt_ppn = (paddr_t)pgt >> RISCV_PGSHIFT;
+
+	assert(!cpu_mmu_enabled());
 
-	assert(asid & g_asid == asid);
 	satp |= SHIFT_U64(asid, RISCV_SATP_ASID_SHIFT);
 	satp |= SHIFT_U64(RISCV_SATP_MODE, RISCV_SATP_MODE_SHIFT);
 	satp |= pgt_ppn;
@@ -288,7 +289,7 @@ static void core_init_mmu_prtn_ta(struct mmu_partition *prtn)
 {
 	unsigned int core = 0;
 
-	assert(user_va_idx != -1);
+	assert(core_mmu_user_va_range_is_defined());
 
 	memset(prtn->user_pgts, 0, CFG_NUM_THREADS * RISCV_MMU_PGT_SIZE);
 	for (core = 0; core < CFG_TEE_CORE_NB_CORE; core++)
@@ -296,7 +297,7 @@ static void core_init_mmu_prtn_ta(struct mmu_partition *prtn)
 }
 
 static void core_init_mmu_prtn_tee(struct mmu_partition *prtn,
-				   struct tee_mmap_region *mm)
+				   struct memory_map *mem_map)
 {
 	size_t n = 0;
 	void *pgt = core_mmu_get_root_pgt_va(prtn);
@@ -306,9 +307,9 @@ static void core_init_mmu_prtn_tee(struct mmu_partition *prtn,
 	memset(pgt, 0, RISCV_MMU_PGT_SIZE);
 	memset(prtn->pool_pgts, 0, RISCV_MMU_MAX_PGTS * RISCV_MMU_PGT_SIZE);
 
-	for (n = 0; !core_mmap_is_end_of_table(mm + n); n++)
-		if (!core_mmu_is_dynamic_vaspace(mm + n))
-			core_mmu_map_region(prtn, mm + n);
+	for (n = 0; n < mem_map->count; n++)
+		if (!core_mmu_is_dynamic_vaspace(mem_map->map + n))
+			core_mmu_map_region(prtn, mem_map->map + n);
 
 	/*
 	 * Primary mapping table is ready at index `get_core_pos()`
@@ -322,6 +323,27 @@ static void core_init_mmu_prtn_tee(struct mmu_partition *prtn,
 	}
 }
 
+/*
+ * Given an entry that points to a table.
+ * If mmu is disabled, returns the pa of pointed table.
+ * If mmu is enabled, returns the va of pointed table.
+ * returns NULL otherwise.
+ */
+static struct mmu_pgt *core_mmu_xlat_table_entry_pa2va(struct mmu_pte *pte,
+						       struct mmu_pgt *pgt)
+{
+	if (core_mmu_entry_is_invalid(pte) ||
+	    core_mmu_entry_is_leaf(pte))
+		return NULL;
+
+	if (!cpu_mmu_enabled())
+		return (struct mmu_pgt *)pte_to_pa(pte);
+
+	return phys_to_virt(pte_to_pa(pte),
+			    MEM_AREA_TEE_RAM_RW_DATA,
+			    sizeof(*pgt));
+}
+
 void tlbi_va_range(vaddr_t va, size_t len,
 		   size_t granule)
 {
@@ -428,7 +450,7 @@ void asid_free(unsigned int asid)
 	if (asid) {
 		int i = asid - 1;
 
-		assert(i < RISCV_MMU_ASID_WIDTH && bit_test(&g_asid, i));
+		assert(i < RISCV_MMU_ASID_WIDTH && bit_test(g_asid, i));
 		bit_clear(g_asid, i);
 	}
 
@@ -444,6 +466,7 @@ bool arch_va2pa_helper(void *va, paddr_t *pa)
 	int level = 0;
 	unsigned int idx = 0;
 	struct mmu_partition *prtn = core_mmu_get_prtn();
+	vaddr_t offset_mask = 0;
 
 	assert(pa);
 
@@ -457,14 +480,13 @@ bool arch_va2pa_helper(void *va, paddr_t *pa)
 			thread_unmask_exceptions(exceptions);
 			return false;
 		} else if (core_mmu_entry_is_leaf(pte)) {
-			*pa = pte_to_pa(pte) |
-			      (vaddr & (BIT64(RISCV_PGSHIFT) - 1));
+			offset_mask = CORE_MMU_PAGE_OFFSET_MASK(level);
+			*pa = pte_to_pa(pte) | (vaddr & offset_mask);
 			thread_unmask_exceptions(exceptions);
 			return true;
 		}
 
-		pgt = phys_to_virt(pte_to_pa(pte),
-				   MEM_AREA_TEE_RAM_RW_DATA, sizeof(*pgt));
+		pgt = core_mmu_xlat_table_entry_pa2va(pte, pgt);
 	}
 
 	thread_unmask_exceptions(exceptions);
@@ -507,8 +529,7 @@ bool core_mmu_find_table(struct mmu_partition *prtn, vaddr_t va,
 			ret = true;
 			goto out;
 		}
-		pgt = phys_to_virt(pte_to_pa(pte),
-				   MEM_AREA_TEE_RAM_RW_DATA, sizeof(*pgt));
+		pgt = core_mmu_xlat_table_entry_pa2va(pte, pgt);
 		if (!pgt)
 			goto out;
 		va_base += SHIFT_U64(idx, CORE_MMU_SHIFT_OF_LEVEL(level));
@@ -526,6 +547,7 @@ bool core_mmu_entry_to_finer_grained(struct core_mmu_table_info *tbl_info,
 	struct mmu_pte *pte = NULL;
 	struct mmu_partition *prtn = core_mmu_get_prtn();
 	unsigned long ptp = 0;
+	paddr_t pgt_pa = 0;
 
 	if (!core_mmu_level_in_range(tbl_info->level))
 		return false;
@@ -538,7 +560,12 @@ bool core_mmu_entry_to_finer_grained(struct core_mmu_table_info *tbl_info,
 		if (!pgt)
 			return false;
 
-		ptp = core_mmu_ptp_create(pa_to_ppn((paddr_t)pgt));
+		if (cpu_mmu_enabled())
+			pgt_pa = virt_to_phys(pgt);
+		else
+			pgt_pa = (paddr_t)pgt;
+
+		ptp = core_mmu_ptp_create(pa_to_ppn(pgt_pa));
 		core_mmu_entry_set(pte, ptp);
 	}
 
@@ -610,7 +637,7 @@ core_mmu_get_user_mapping_entry(struct mmu_partition *prtn)
 {
 	struct mmu_pgt *pgt = core_mmu_get_root_pgt_va(prtn);
 
-	assert(user_va_idx != -1);
+	assert(core_mmu_user_va_range_is_defined());
 
 	return core_mmu_table_get_entry(pgt, user_va_idx);
 }
@@ -642,9 +669,14 @@ void core_mmu_set_user_map(struct core_mmu_user_map *map)
 	thread_unmask_exceptions(exceptions);
 }
 
+bool core_mmu_user_va_range_is_defined(void)
+{
+	return user_va_idx != -1;
+}
+
 void core_mmu_get_user_va_range(vaddr_t *base, size_t *size)
 {
-	assert(user_va_idx != -1);
+	assert(core_mmu_user_va_range_is_defined());
 
 	if (base)
 		*base = SHIFT_U64(user_va_idx, CORE_MMU_BASE_TABLE_SHIFT);
@@ -705,13 +737,13 @@ bool core_mmu_user_mapping_is_active(void)
 	return ret;
 }
 
-void core_init_mmu_prtn(struct mmu_partition *prtn, struct tee_mmap_region *mm)
+void core_init_mmu_prtn(struct mmu_partition *prtn, struct memory_map *mem_map)
 {
-	core_init_mmu_prtn_tee(prtn, mm);
+	core_init_mmu_prtn_tee(prtn, mem_map);
 	core_init_mmu_prtn_ta(prtn);
 }
 
-void core_init_mmu(struct tee_mmap_region *mm)
+void core_init_mmu(struct memory_map *mem_map)
 {
 	uint64_t max_va = 0;
 	size_t n = 0;
@@ -720,10 +752,10 @@ void core_init_mmu(struct tee_mmap_region *mm)
 			    sizeof(pool_pgts));
 
 	/* Initialize default pagetables */
-	core_init_mmu_prtn_tee(&default_partition, mm);
+	core_init_mmu_prtn_tee(&default_partition, mem_map);
 
-	for (n = 0; !core_mmap_is_end_of_table(mm + n); n++) {
-		vaddr_t va_end = mm[n].va + mm[n].size - 1;
+	for (n = 0; n < mem_map->count; n++) {
+		vaddr_t va_end = mem_map->map[n].va + mem_map->map[n].size - 1;
 
 		if (va_end > max_va)
 			max_va = va_end;
