@@ -736,17 +736,14 @@ static void bsec_dt_otp_nsec_access(void *fdt, int bsec_node)
 		panic();
 
 	fdt_for_each_subnode(bsec_subnode, fdt, bsec_node) {
-		unsigned int reg_offset = 0;
-		unsigned int reg_size = 0;
+		paddr_t reg_offset = 0;
+		size_t reg_size = 0;
 		unsigned int otp_id = 0;
 		unsigned int i = 0;
 		size_t size = 0;
 
-		reg_offset = fdt_reg_base_address(fdt, bsec_subnode);
-		reg_size = fdt_reg_size(fdt, bsec_subnode);
-
-		assert(reg_offset != DT_INFO_INVALID_REG &&
-		       reg_size != DT_INFO_INVALID_REG_SIZE);
+		if (fdt_reg_info(fdt, bsec_subnode, &reg_offset, &reg_size))
+			panic();
 
 		otp_id = reg_offset / sizeof(uint32_t);
 
@@ -837,8 +834,8 @@ static void save_dt_nvmem_layout(void *fdt, int bsec_node)
 		panic();
 
 	fdt_for_each_subnode(node, fdt, bsec_node) {
-		unsigned int reg_offset = 0;
-		unsigned int reg_length = 0;
+		paddr_t reg_offset = 0;
+		size_t reg_length = 0;
 		const char *string = NULL;
 		const char *s = NULL;
 		int len = 0;
@@ -852,11 +849,7 @@ static void save_dt_nvmem_layout(void *fdt, int bsec_node)
 		layout_cell->phandle = fdt_get_phandle(fdt, node);
 		assert(layout_cell->phandle != (uint32_t)-1);
 
-		reg_offset = fdt_reg_base_address(fdt, node);
-		reg_length = fdt_reg_size(fdt, node);
-
-		if (reg_offset == DT_INFO_INVALID_REG ||
-		    reg_length == DT_INFO_INVALID_REG_SIZE) {
+		if (fdt_reg_info(fdt, node, &reg_offset, &reg_length)) {
 			DMSG("Malformed nvmem %s: ignored", string);
 			continue;
 		}
