@@ -819,9 +819,6 @@ int stm32_i2c_init(struct i2c_handle_s *hi2c,
 
 	i2c_config_analog_filter(hi2c, init_data->analog_filter);
 
-	if (IS_ENABLED(CFG_STM32MP13))
-		stm32_pinctrl_set_secure_cfg(hi2c->pinctrl, true);
-
 	clk_disable(hi2c->clock);
 
 	if (hi2c->pinctrl && pinctrl_apply_state(hi2c->pinctrl))
@@ -1586,9 +1583,6 @@ void stm32_i2c_resume(struct i2c_handle_s *hi2c)
 
 	restore_cfg(hi2c, &hi2c->sec_cfg);
 
-	if (IS_ENABLED(CFG_STM32MP13))
-		stm32_pinctrl_set_secure_cfg(hi2c->pinctrl, true);
-
 	hi2c->i2c_state = I2C_STATE_READY;
 }
 
@@ -1614,7 +1608,7 @@ static TEE_Result stm32_get_i2c_dev(struct dt_pargs *args, void *data,
 	struct stm32_i2c_dev *stm32_i2c_dev = NULL;
 	paddr_t addr = 0;
 
-	addr = fdt_reg_base_address(args->fdt, args->phandle_node);
+	addr = fdt_reg_base_address(args->fdt, args->consumer_node);
 	if (addr == DT_INFO_INVALID_REG) {
 		DMSG("Can't get device I2C address");
 		return TEE_ERROR_GENERIC;
