@@ -124,30 +124,52 @@
 #define STM32MP25_RIFSC_OTFDEC2_ID		126
 #define STM32MP25_RIFSC_IAC_ID			127
 
+/* RIF-aware IPs */
+#define STM32MP25_RIFSC_PWR_ID			155
+#define STM32MP25_RIFSC_GPIOA_ID		160
+#define STM32MP25_RIFSC_GPIOB_ID		161
+#define STM32MP25_RIFSC_GPIOC_ID		162
+#define STM32MP25_RIFSC_GPIOD_ID		163
+#define STM32MP25_RIFSC_GPIOE_ID		164
+#define STM32MP25_RIFSC_GPIOF_ID		165
+#define STM32MP25_RIFSC_GPIOG_ID		166
+#define STM32MP25_RIFSC_GPIOH_ID		167
+#define STM32MP25_RIFSC_GPIOI_ID		168
+#define STM32MP25_RIFSC_GPIOJ_ID		169
+#define STM32MP25_RIFSC_GPIOK_ID		170
+#define STM32MP25_RIFSC_GPIOZ_ID		171
+
 /* Global lock bindings */
 #define RIFSC_RIMU_GLOCK			1
 #define RIFSC_RISUP_GLOCK			2
 
 /* masters ID */
-#define RIMU_ID(idx)		(idx)
+#define RIMU_ID_OFFSET		200
+#define RIMU_ID(idx)		((idx) + RIMU_ID_OFFSET)
 
 /* master configuration modes */
 #define RIF_CIDSEL_P	0x0 /* config from RISUP */
 #define RIF_CIDSEL_M	0x1 /* config from RIMU */
 
-#define RIFSC_RIMC_MODE_SHIFT		2
-#define RIFSC_RIMC_MCID_SHIFT		4
-#define RIFSC_RIMC_MSEC_SHIFT		8
-#define RIFSC_RIMC_MPRIV_SHIFT		9
-#define RIFSC_RIMC_M_ID_SHIFT		16
-#define RIFSC_RIMC_ATTRx_SHIFT		0
+#define RIMUPROT_RIMC_M_ID_SHIFT	0
+#define RIMUPROT_RIMC_MODE_SHIFT	10
+#define RIMUPROT_RIMC_MCID_SHIFT	12
+#define RIMUPROT_RIMC_MSEC_SHIFT	16
+#define RIMUPROT_RIMC_MPRIV_SHIFT	17
+#define RIMUPROT_RIMC_ATTRx_SHIFT	8
+#define RIMUPROT_RIMC_M_ID_MASK		GENMASK_32(7, 0)
+#define RIMUPROT_RIMC_MCID_MASK		GENMASK_32(15, 12)
+#define RIMUPROT_RIMC_ATTRx_MASK	(BIT(RIMUPROT_RIMC_MODE_SHIFT) | \
+					 RIMUPROT_RIMC_MCID_MASK | \
+					 BIT(RIMUPROT_RIMC_MSEC_SHIFT) | \
+					 BIT(RIMUPROT_RIMC_MPRIV_SHIFT))
 
 #define RIMUPROT(rimuid, mcid, msec, mpriv, mode) \
-	(((rimuid) << RIFSC_RIMC_M_ID_SHIFT) | \
-	 ((mpriv) << RIFSC_RIMC_MPRIV_SHIFT) | \
-	 ((msec) << RIFSC_RIMC_MSEC_SHIFT) | \
-	 ((mcid) << RIFSC_RIMC_MCID_SHIFT) | \
-	 ((mode) << RIFSC_RIMC_MODE_SHIFT))
+	(((rimuid) << RIMUPROT_RIMC_M_ID_SHIFT) | \
+	 ((mpriv) << RIMUPROT_RIMC_MPRIV_SHIFT) | \
+	 ((msec) << RIMUPROT_RIMC_MSEC_SHIFT) | \
+	 ((mcid) << RIMUPROT_RIMC_MCID_SHIFT) | \
+	 ((mode) << RIMUPROT_RIMC_MODE_SHIFT))
 
 #endif /* _DT_BINDINGS_FIREWALL_STM32MP25_RIFSC_H */
 
