@@ -89,12 +89,14 @@ struct dt_driver_provider;
  * struct dt_pargs - Devicetree phandle arguments
  * @fdt: Device-tree to work on
  * @phandle_node: Node pointed by the specifier phandle
+ * @consumer_node: Node of the consumer requesting this device
  * @args_count: Count of cells for the device
  * @args: Device consumer specifiers
  */
 struct dt_pargs {
 	const void *fdt;
 	int phandle_node;
+	int consumer_node;
 	int args_count;
 	uint32_t args[];
 };
