@@ -57,7 +57,6 @@
 #ifndef __ASSEMBLER__
 /*
  * Memory area type:
- * MEM_AREA_END:      Reserved, marks the end of a table of mapping areas.
  * MEM_AREA_TEE_RAM:  core RAM (read/write/executable, secure, reserved to TEE)
  * MEM_AREA_TEE_RAM_RX:  core private read-only/executable memory (secure)
  * MEM_AREA_TEE_RAM_RO:  core private read-only/non-executable memory (secure)
@@ -69,7 +68,6 @@
  * MEM_AREA_TEE_COHERENT: teecore coherent RAM (secure, reserved to TEE)
  * MEM_AREA_TEE_ASAN: core address sanitizer RAM (secure, reserved to TEE)
  * MEM_AREA_IDENTITY_MAP_RX: core identity mapped r/o executable memory (secure)
- * MEM_AREA_TA_RAM:   Secure RAM where teecore loads/exec TA instances.
  * MEM_AREA_NSEC_SHM: NonSecure shared RAM between NSec and TEE.
  * MEM_AREA_NEX_NSEC_SHM: nexus non-secure shared RAM between NSec and TEE.
  * MEM_AREA_RAM_NSEC: NonSecure RAM storing data
@@ -88,8 +86,7 @@
  * MEM_AREA_MAXTYPE:  lower invalid 'type' value
  */
 enum teecore_memtypes {
-	MEM_AREA_END = 0,
-	MEM_AREA_TEE_RAM,
+	MEM_AREA_TEE_RAM = 1,
 	MEM_AREA_TEE_RAM_RX,
 	MEM_AREA_TEE_RAM_RO,
 	MEM_AREA_TEE_RAM_RW,
@@ -100,7 +97,6 @@ enum teecore_memtypes {
 	MEM_AREA_TEE_COHERENT,
 	MEM_AREA_TEE_ASAN,
 	MEM_AREA_IDENTITY_MAP_RX,
-	MEM_AREA_TA_RAM,
 	MEM_AREA_NSEC_SHM,
 	MEM_AREA_NEX_NSEC_SHM,
 	MEM_AREA_RAM_NSEC,
@@ -124,7 +120,6 @@ enum teecore_memtypes {
 static inline const char *teecore_memtype_name(enum teecore_memtypes type)
 {
 	static const char * const names[] = {
-		[MEM_AREA_END] = "END",
 		[MEM_AREA_TEE_RAM] = "TEE_RAM_RWX",
 		[MEM_AREA_TEE_RAM_RX] = "TEE_RAM_RX",
 		[MEM_AREA_TEE_RAM_RO] = "TEE_RAM_RO",
@@ -136,7 +131,6 @@ static inline const char *teecore_memtype_name(enum teecore_memtypes type)
 		[MEM_AREA_TEE_ASAN] = "TEE_ASAN",
 		[MEM_AREA_IDENTITY_MAP_RX] = "IDENTITY_MAP_RX",
 		[MEM_AREA_TEE_COHERENT] = "TEE_COHERENT",
-		[MEM_AREA_TA_RAM] = "TA_RAM",
 		[MEM_AREA_NSEC_SHM] = "NSEC_SHM",
 		[MEM_AREA_NEX_NSEC_SHM] = "NEX_NSEC_SHM",
 		[MEM_AREA_RAM_NSEC] = "RAM_NSEC",
@@ -277,9 +271,6 @@ struct core_mmu_phys_mem {
 #define phys_mem_map_end \
 	SCATTERED_ARRAY_END(phys_mem_map, struct core_mmu_phys_mem)
 
-/* Physical Secure DDR pool */
-extern tee_mm_pool_t tee_mm_sec_ddr;
-
 /* Virtual memory pool for core mappings */
 extern tee_mm_pool_t core_virt_mem_pool;
 
@@ -304,6 +295,11 @@ extern const unsigned long core_mmu_tee_load_pa;
 
 void core_init_mmu_map(unsigned long seed, struct core_mmu_config *cfg);
 void core_init_mmu_regs(struct core_mmu_config *cfg);
+/*
+ * Copy static memory map from temporary boot_mem to heap when CFG_BOOT_MEM
+ * is enabled.
+ */
+void core_mmu_save_mem_map(void);
 
 /* Arch specific function to help optimizing 1 MMU xlat table */
 bool core_mmu_prefer_tee_ram_at_top(paddr_t paddr);
@@ -554,12 +550,22 @@ void core_mmu_unmap_pages(vaddr_t vstart, size_t num_pages);
  */
 bool core_mmu_user_mapping_is_active(void);
 
+/*
+ * core_mmu_user_va_range_is_defined() - check if user va range is defined
+ * @returns true if a user VA space is defined, false if not.
+ */
+bool core_mmu_user_va_range_is_defined(void);
+
 /*
  * core_mmu_mattr_is_ok() - Check that supplied mem attributes can be used
  * @returns true if the attributes can be used, false if not.
  */
 bool core_mmu_mattr_is_ok(uint32_t mattr);
 
+TEE_Result core_mmu_for_each_map(void *ptr,
+				 TEE_Result (*fn)(struct tee_mmap_region *map,
+						  void *ptr));
+
 void core_mmu_get_mem_by_type(enum teecore_memtypes type, vaddr_t *s,
 			      vaddr_t *e);
 
@@ -625,7 +631,7 @@ void core_mmu_set_discovered_nsec_ddr(struct core_mmu_phys_mem *start,
 #endif
 
 /* Initialize MMU partition */
-void core_init_mmu_prtn(struct mmu_partition *prtn, struct tee_mmap_region *mm);
+void core_init_mmu_prtn(struct mmu_partition *prtn, struct memory_map *mem_map);
 
 unsigned int asid_alloc(void);
 void asid_free(unsigned int asid);
@@ -644,12 +650,10 @@ void core_mmu_set_default_prtn(void);
 void core_mmu_set_default_prtn_tbl(void);
 #endif
 
-void core_mmu_init_virtualization(void);
+/* Initialize physical memory pool */
+void core_mmu_init_phys_mem(void);
 
-/* init some allocation pools */
-void core_mmu_init_ta_ram(void);
-
-void core_init_mmu(struct tee_mmap_region *mm);
+void core_init_mmu(struct memory_map *mem_map);
 
 void core_mmu_set_info_table(struct core_mmu_table_info *tbl_info,
 			     unsigned int level, vaddr_t va_base, void *table);
@@ -660,11 +664,6 @@ void core_mmu_map_region(struct mmu_partition *prtn,
 
 bool arch_va2pa_helper(void *va, paddr_t *pa);
 
-static inline bool core_mmap_is_end_of_table(const struct tee_mmap_region *mm)
-{
-	return mm->type == MEM_AREA_END;
-}
-
 static inline bool core_mmu_check_end_pa(paddr_t pa, size_t len)
 {
 	paddr_t end_pa = 0;
@@ -696,13 +695,6 @@ void core_mmu_set_secure_memory(paddr_t base, size_t size);
  */
 void core_mmu_get_secure_memory(paddr_t *base, paddr_size_t *size);
 
-/*
- * core_mmu_get_ta_range() - get physical memory range reserved for TAs
- * @base: [out] range base address ref or NULL
- * @size: [out] range size ref or NULL
- */
-void core_mmu_get_ta_range(paddr_t *base, size_t *size);
-
 #endif /*__ASSEMBLER__*/
 
 #endif /* __MM_CORE_MMU_H */
