@@ -29,6 +29,12 @@ TEE_Result tee_do_cipher_update(void *ctx, uint32_t algo,
  */
 void plat_prng_add_jitter_entropy(enum crypto_rng_src sid, unsigned int *pnum);
 
+/*
+ * plat_rng_init() - Initialize RNG support.
+ * If not overridden with a platform implementation, the function defaults
+ * to __plat_rng_init() that implements a weak seeding of the RNG.
+ */
 void plat_rng_init(void);
+void __plat_rng_init(void);
 
 #endif
