@@ -125,8 +125,8 @@ void *__GP11_TEE_Realloc(void *buffer, uint32_t newSize);
 
 void TEE_Free(void *buffer);
 
-void *TEE_MemMove(void *dest, const void *src, size_t size);
-void *__GP11_TEE_MemMove(void *dest, const void *src, uint32_t size);
+void TEE_MemMove(void *dest, const void *src, size_t size);
+void __GP11_TEE_MemMove(void *dest, const void *src, uint32_t size);
 
 /*
  * Note: TEE_MemCompare() has a constant-time implementation (execution time
